@@ -1,7 +1,7 @@
 #![deny(unused_must_use)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
-//! Raft is a distributed consensus algorithm widely used nowadays
+//! Raft is a distributed consensus algorithm widely used recently
 //! to build distributed applications like etcd.
 //! However, while it is even understandable than notorious Paxos algorithm
 //! it is still difficult to implement correct and efficient implementation.
@@ -12,6 +12,7 @@
 //! and snapshot copying is very efficient.
 
 use anyhow::anyhow;
+use anyhow::Result;
 use async_trait::async_trait;
 use bytes::Bytes;
 use derive_builder::Builder;
@@ -35,17 +36,15 @@ mod quorum_join;
 mod server;
 #[cfg(feature = "simple")]
 #[cfg_attr(docsrs, doc(cfg(feature = "simple")))]
-/// Simplified RaftApp trait.
+/// Simplified `RaftApp`.
 pub mod simple;
-/// The snapshot abstraction and some basic implementations.
-pub mod snapshot;
-/// The abstraction for the backing storage and some implementations.
+mod snapshot;
+/// The abstraction of the log storage and some implementations.
 pub mod storage;
 mod thread;
 mod thread_drop;
 
 use ack::Ack;
-use snapshot::SnapshotTag;
 use storage::RaftStorage;
 use tonic::transport::Endpoint;
 
@@ -66,17 +65,17 @@ pub use crate::proto_compiled::raft_client::RaftClient;
 
 use storage::{Ballot, Entry};
 
-/// Plan to make a new snapshot.
+/// Decision to make a new snapshot.
 pub enum MakeSnapshot {
     /// No snapshot will be made.
     None,
     /// Copy snapshot will be made.
-    CopySnapshot(SnapshotTag),
+    CopySnapshot,
     /// Fold snapshot will be made.
     FoldSnapshot,
 }
 
-/// The abstraction for user-defined application runs on the RaftCore.
+/// The abstraction of user-defined application.
 ///
 /// Note about the error handling:
 /// In Raft, the great rule is the same log should result in the same state.
@@ -88,7 +87,7 @@ pub enum MakeSnapshot {
 pub trait RaftApp: Sync + Send + 'static {
     /// Process read request.
     /// This operation should **not** change the state of the application.
-    async fn process_read(&self, request: &[u8]) -> anyhow::Result<Vec<u8>>;
+    async fn process_read(&self, request: &[u8]) -> Result<Vec<u8>>;
 
     /// Process write request.
     /// This may change the state of the application.
@@ -100,41 +99,38 @@ pub trait RaftApp: Sync + Send + 'static {
         &self,
         request: &[u8],
         entry_index: Index,
-    ) -> anyhow::Result<(Vec<u8>, MakeSnapshot)>;
+    ) -> Result<(Vec<u8>, MakeSnapshot)>;
 
     /// Special type of process_write but when the entry is a snapshot entry.
     /// Snapshot is None when apply_index is 1 which is the most youngest snapshot.
-    async fn install_snapshot(
-        &self,
-        snapshot: Option<&SnapshotTag>,
-        snapshot_index: Index,
-    ) -> anyhow::Result<()>;
+    async fn install_snapshot(&self, snapshot: Option<Index>) -> Result<()>;
 
     /// This function is called from the compaction thread.
     /// It should return new snapshot from accumulative computation with the old_snapshot and the subsequent log entries.
     async fn fold_snapshot(
         &self,
-        old_snapshot: Option<&SnapshotTag>,
+        old_snapshot: Option<Index>,
         requests: Vec<&[u8]>,
         snapshot_index: Index,
-    ) -> anyhow::Result<SnapshotTag>;
+    ) -> Result<()>;
 
-    /// Make a snapshot resource and return the tag.
-    async fn save_snapshot(
-        &self,
-        st: snapshot::SnapshotStream,
-        snapshot_index: Index,
-    ) -> anyhow::Result<SnapshotTag>;
+    /// Make a snapshot resource from an in-coming `SnapshotStream`.
+    async fn save_snapshot(&self, st: SnapshotStream, snapshot_index: Index) -> Result<()>;
 
-    /// Make a snapshot stream from a snapshot resource bound to the tag.
-    async fn open_snapshot(&self, x: &SnapshotTag) -> snapshot::SnapshotStream;
+    /// Open a `SnapshotStream` from a snapshot resource.
+    async fn open_snapshot(&self, x: Index) -> Result<SnapshotStream>;
 
-    /// Delete a snapshot resource bound to the tag.
-    async fn delete_snapshot(&self, x: &SnapshotTag) -> anyhow::Result<()>;
+    /// Delete a snapshot resource.
+    async fn delete_snapshot(&self, x: Index) -> Result<()>;
 }
 
+/// The core-level stream type. It is just a stream of byte chunks.
+/// The length of each chunk may be different.
+pub type SnapshotStream =
+    std::pin::Pin<Box<dyn futures::stream::Stream<Item = anyhow::Result<Bytes>> + Send>>;
+
 type Term = u64;
-/// Log index.
+/// Log entry index.
 pub type Index = u64;
 #[derive(Clone, Copy, Eq)]
 struct Clock {
@@ -316,9 +312,7 @@ impl RaftCore {
             .unwrap();
         r
     }
-    async fn find_last_membership<S: RaftStorage>(
-        storage: &S,
-    ) -> anyhow::Result<(Index, HashSet<Id>)> {
+    async fn find_last_membership<S: RaftStorage>(storage: &S) -> Result<(Index, HashSet<Id>)> {
         let last = storage.get_last_index().await?;
         let mut ret = (0, HashSet::new());
         for i in (1..=last).rev() {
@@ -337,7 +331,7 @@ impl RaftCore {
         }
         Ok(ret)
     }
-    async fn init_cluster(self: &Arc<Self>) -> anyhow::Result<()> {
+    async fn init_cluster(self: &Arc<Self>) -> Result<()> {
         let snapshot = Entry {
             prev_clock: Clock { term: 0, index: 0 },
             this_clock: Clock { term: 0, index: 1 },
@@ -374,7 +368,7 @@ impl RaftCore {
         self: &Arc<Self>,
         membership: &HashSet<Id>,
         index: Index,
-    ) -> anyhow::Result<()> {
+    ) -> Result<()> {
         log::info!("change membership to {:?}", membership);
         self.cluster
             .write()
@@ -443,11 +437,7 @@ fn into_out_stream(
 }
 // Replication
 impl RaftCore {
-    async fn change_membership(
-        self: &Arc<Self>,
-        command: Bytes,
-        index: Index,
-    ) -> anyhow::Result<()> {
+    async fn change_membership(self: &Arc<Self>, command: Bytes, index: Index) -> Result<()> {
         match Command::deserialize(&command) {
             Command::Snapshot { membership } => {
                 self.set_membership(&membership, index).await?;
@@ -464,7 +454,7 @@ impl RaftCore {
         self.safe_term.fetch_max(term, Ordering::SeqCst);
     }
     // Leader calls this fucntion to append new entry to its log.
-    async fn queue_entry(self: &Arc<Self>, command: Bytes, ack: Option<Ack>) -> anyhow::Result<()> {
+    async fn queue_entry(self: &Arc<Self>, command: Bytes, ack: Option<Ack>) -> Result<()> {
         let term = self.load_ballot().await?.cur_term;
         // safe_term is a term that noop entry is successfully committed.
         let cur_safe_term = self.safe_term.load(Ordering::SeqCst);
@@ -487,7 +477,7 @@ impl RaftCore {
         Ok(())
     }
     // Follower calls this function when it receives entries from the leader.
-    async fn queue_received_entry(self: &Arc<Self>, mut req: LogStream) -> anyhow::Result<bool> {
+    async fn queue_received_entry(self: &Arc<Self>, mut req: LogStream) -> Result<bool> {
         let mut prev_clock = Clock {
             term: req.prev_log_term,
             index: req.prev_log_index,
@@ -524,11 +514,7 @@ impl RaftCore {
         }
         Ok(true)
     }
-    async fn prepare_replication_stream(
-        self: &Arc<Self>,
-        l: Index,
-        r: Index,
-    ) -> anyhow::Result<LogStream> {
+    async fn prepare_replication_stream(self: &Arc<Self>, l: Index, r: Index) -> Result<LogStream> {
         let head = self.log.storage.get_entry(l).await?.unwrap();
         let Clock {
             term: prev_log_term,
@@ -562,7 +548,7 @@ impl RaftCore {
             entries: Box::pin(st),
         })
     }
-    async fn advance_replication(self: &Arc<Self>, follower_id: Id) -> anyhow::Result<bool> {
+    async fn advance_replication(self: &Arc<Self>, follower_id: Id) -> Result<bool> {
         let peer = self
             .cluster
             .read()
@@ -604,7 +590,7 @@ impl RaftCore {
             .prepare_replication_stream(old_progress.next_index, old_progress.next_index + n)
             .await?;
 
-        let res: anyhow::Result<_> = async {
+        let res: Result<_> = async {
             let endpoint =
                 Endpoint::from(follower_id.uri().clone()).connect_timeout(Duration::from_secs(5));
             let mut conn = RaftClient::connect(endpoint).await?;
@@ -649,7 +635,7 @@ impl RaftCore {
 
         Ok(true)
     }
-    async fn find_new_agreement(&self) -> anyhow::Result<Index> {
+    async fn find_new_agreement(&self) -> Result<Index> {
         let mut match_indices = vec![];
 
         // In leader stepdown, leader is out of the membership
@@ -670,7 +656,7 @@ impl RaftCore {
 }
 // Snapshot
 impl RaftCore {
-    async fn fetch_snapshot(&self, snapshot_index: Index, to: Id) -> anyhow::Result<()> {
+    async fn fetch_snapshot(&self, snapshot_index: Index, to: Id) -> Result<()> {
         let endpoint = Endpoint::from(to.uri().clone()).connect_timeout(Duration::from_secs(5));
 
         let mut conn = RaftClient::connect(endpoint).await?;
@@ -680,29 +666,20 @@ impl RaftCore {
         let res = conn.get_snapshot(req).await?;
         let out_stream = res.into_inner();
         let in_stream = Box::pin(snapshot::into_in_stream(out_stream));
-        let tag = self.app.save_snapshot(in_stream, snapshot_index).await?;
-        self.log.storage.put_tag(snapshot_index, tag).await?;
+        self.app.save_snapshot(in_stream, snapshot_index).await?;
         Ok(())
     }
-    async fn make_snapshot_stream(
-        &self,
-        snapshot_index: Index,
-    ) -> anyhow::Result<Option<snapshot::SnapshotStream>> {
-        let tag = self.log.storage.get_tag(snapshot_index).await?;
-        if tag.is_none() {
-            return Ok(None);
-        }
-        let tag = tag.unwrap();
-        let st = self.app.open_snapshot(&tag).await;
+    async fn make_snapshot_stream(&self, snapshot_index: Index) -> Result<Option<SnapshotStream>> {
+        let st = self.app.open_snapshot(snapshot_index).await?;
         Ok(Some(st))
     }
 }
 // Election
 impl RaftCore {
-    async fn save_ballot(&self, v: Ballot) -> anyhow::Result<()> {
+    async fn save_ballot(&self, v: Ballot) -> Result<()> {
         self.log.storage.save_ballot(v).await
     }
-    async fn load_ballot(&self) -> anyhow::Result<Ballot> {
+    async fn load_ballot(&self) -> Result<Ballot> {
         self.log.storage.load_ballot().await
     }
     async fn detect_election_timeout(&self) -> bool {
@@ -718,7 +695,7 @@ impl RaftCore {
         candidate_last_log_clock: Clock,
         force_vote: bool,
         pre_vote: bool,
-    ) -> anyhow::Result<bool> {
+    ) -> Result<bool> {
         let allow_side_effects = !pre_vote;
 
         if !force_vote {
@@ -804,7 +781,7 @@ impl RaftCore {
         aim_term: Term,
         force_vote: bool,
         pre_vote: bool,
-    ) -> anyhow::Result<bool> {
+    ) -> Result<bool> {
         let (others, remaining) = {
             let membership = self.cluster.read().await.membership.clone();
             let n = membership.len();
@@ -853,7 +830,7 @@ impl RaftCore {
                     // We recommend the Pre-Vote extension in deployments that would benefit from additional robustness.
                     pre_vote,
                 };
-                let res: anyhow::Result<_> = async {
+                let res: Result<_> = async {
                     let endpoint =
                         Endpoint::from(endpoint.uri().clone()).timeout(Duration::from_secs(5));
                     let mut conn = RaftClient::connect(endpoint).await?;
@@ -870,7 +847,7 @@ impl RaftCore {
         let ok = quorum_join::quorum_join(remaining, vote_requests).await;
         Ok(ok)
     }
-    async fn after_votes(self: &Arc<Self>, aim_term: Term, ok: bool) -> anyhow::Result<()> {
+    async fn after_votes(self: &Arc<Self>, aim_term: Term, ok: bool) -> Result<()> {
         if ok {
             log::info!("got enough votes from the cluster. promoted to leader");
 
@@ -898,7 +875,7 @@ impl RaftCore {
         }
         Ok(())
     }
-    async fn try_promote(self: &Arc<Self>, force_vote: bool) -> anyhow::Result<()> {
+    async fn try_promote(self: &Arc<Self>, force_vote: bool) -> Result<()> {
         // Pre-Vote phase.
         let pre_aim_term = {
             let _ballot_guard = self.vote_token.acquire().await;
@@ -952,7 +929,7 @@ impl RaftCore {
 
         Ok(())
     }
-    async fn send_heartbeat(&self, follower_id: Id) -> anyhow::Result<()> {
+    async fn send_heartbeat(&self, follower_id: Id) -> Result<()> {
         let endpoint = Endpoint::from(follower_id.uri().clone()).timeout(Duration::from_secs(5));
         let req = {
             let term = self.load_ballot().await?.cur_term;
@@ -988,7 +965,7 @@ impl RaftCore {
         leader_id: Id,
         leader_term: Term,
         leader_commit: Index,
-    ) -> anyhow::Result<()> {
+    ) -> Result<()> {
         let ballot_guard = self.vote_token.acquire().await;
         let mut ballot = self.load_ballot().await?;
         if leader_term < ballot.cur_term {
@@ -1106,7 +1083,7 @@ impl Log {
             apply_error_seq: 0.into(),
         }
     }
-    async fn get_last_log_index(&self) -> anyhow::Result<Index> {
+    async fn get_last_log_index(&self) -> Result<Index> {
         self.storage.get_last_index().await
     }
     fn get_snapshot_index(&self) -> Index {
@@ -1117,7 +1094,7 @@ impl Log {
         command: Bytes,
         ack: Option<Ack>,
         term: Term,
-    ) -> anyhow::Result<Index> {
+    ) -> Result<Index> {
         let _token = self.append_token.acquire().await;
 
         let cur_last_log_index = self.storage.get_last_index().await?;
@@ -1149,7 +1126,7 @@ impl Log {
         entry: Entry,
         sender_id: Id,
         core: Arc<RaftCore>,
-    ) -> anyhow::Result<TryInsertResult> {
+    ) -> Result<TryInsertResult> {
         let _token = self.append_token.acquire().await;
 
         let Clock {
@@ -1240,22 +1217,18 @@ impl Log {
             Ok(TryInsertResult::Inserted)
         }
     }
-    async fn insert_entry(&self, e: Entry) -> anyhow::Result<()> {
+    async fn insert_entry(&self, e: Entry) -> Result<()> {
         self.storage.insert_entry(e.this_clock.index, e).await?;
         Ok(())
     }
-    async fn insert_snapshot(&self, e: Entry) -> anyhow::Result<()> {
+    async fn insert_snapshot(&self, e: Entry) -> Result<()> {
         let new_snapshot_index = e.this_clock.index;
         self.storage.insert_entry(e.this_clock.index, e).await?;
         self.snapshot_index
             .fetch_max(new_snapshot_index, Ordering::SeqCst);
         Ok(())
     }
-    async fn advance_commit_index(
-        &self,
-        new_agreement: Index,
-        core: Arc<RaftCore>,
-    ) -> anyhow::Result<()> {
+    async fn advance_commit_index(&self, new_agreement: Index, core: Arc<RaftCore>) -> Result<()> {
         let _token = self.commit_token.acquire().await;
 
         let old_agreement = self.commit_index.load(Ordering::SeqCst);
@@ -1308,7 +1281,7 @@ impl Log {
         self.commit_notify.notify_one();
         Ok(())
     }
-    async fn advance_last_applied(&self, raft_core: Arc<RaftCore>) -> anyhow::Result<()> {
+    async fn advance_last_applied(&self, raft_core: Arc<RaftCore>) -> Result<()> {
         let (apply_index, apply_entry, command) = {
             let apply_index = self.last_applied.load(Ordering::SeqCst) + 1;
             let mut e = self.storage.get_entry(apply_index).await?.unwrap();
@@ -1317,12 +1290,13 @@ impl Log {
         };
         let ok = match Command::deserialize(&command) {
             Command::Snapshot { membership } => {
-                let tag = self.storage.get_tag(apply_index).await?;
                 log::info!("install app snapshot");
-                let res = raft_core
-                    .app
-                    .install_snapshot(tag.as_ref(), apply_index)
-                    .await;
+                let snapshot = if apply_index == 1 {
+                    None
+                } else {
+                    Some(apply_index)
+                };
+                let res = raft_core.app.install_snapshot(snapshot).await;
                 log::info!("install app snapshot (complete)");
                 let success = res.is_ok();
                 if success {
@@ -1348,33 +1322,22 @@ impl Log {
                         }
 
                         match make_snapshot {
-                            MakeSnapshot::CopySnapshot(new_tag) => {
-                                // Replication proceeds if process_write's succeeded and regardless of the result of snapshot insertion.
+                            MakeSnapshot::CopySnapshot => {
+                                // Now that RaftApp's already made a snapshot resource then we will commit the snapshot entry.
+                                let snapshot_entry = Entry {
+                                    command: Command::serialize(&Command::Snapshot {
+                                        membership: self.applied_membership.lock().await.clone(),
+                                    }),
+                                    ..apply_entry
+                                };
+                                let delay =
+                                    raft_core.config.read().await.snapshot_insertion_delay();
+                                log::info!(
+                                    "copy snapshot is made and will be inserted in {:?}",
+                                    delay
+                                );
 
-                                // Snapshot becomes orphan when this code fails but it is allowed.
-                                let ok = self.storage.put_tag(apply_index, new_tag).await.is_ok();
-                                // Inserting a snapshot entry with its corresponding snapshot tag/resource is not allowed
-                                // because it is assumed that the at least lastest snapshot entry must have a snapshot tag/resource.
-                                if ok {
-                                    let snapshot_entry = Entry {
-                                        command: Command::serialize(&Command::Snapshot {
-                                            membership: self
-                                                .applied_membership
-                                                .lock()
-                                                .await
-                                                .clone(),
-                                        }),
-                                        ..apply_entry
-                                    };
-                                    let delay =
-                                        raft_core.config.read().await.snapshot_insertion_delay();
-                                    log::info!(
-                                        "copy snapshot is made and will be inserted in {:?}",
-                                        delay
-                                    );
-
-                                    let _ = self.snapshot_queue.insert(snapshot_entry, delay).await;
-                                }
+                                let _ = self.snapshot_queue.insert(snapshot_entry, delay).await;
                             }
                             MakeSnapshot::FoldSnapshot => {
                                 tokio::spawn(async move {
@@ -1429,7 +1392,7 @@ impl Log {
         &self,
         new_snapshot_index: Index,
         core: Arc<RaftCore>,
-    ) -> anyhow::Result<()> {
+    ) -> Result<()> {
         assert!(new_snapshot_index <= self.last_applied.load(Ordering::SeqCst));
 
         let _token = self.compaction_token.acquire().await;
@@ -1471,12 +1434,14 @@ impl Log {
                     _ => {}
                 }
             }
-            let base_tag = self.storage.get_tag(base_snapshot_index).await?;
-            let new_tag = core
-                .app
-                .fold_snapshot(base_tag.as_ref(), app_messages, new_snapshot_index)
+            let base_snapshot = if base_snapshot_index == 1 {
+                None
+            } else {
+                Some(base_snapshot_index)
+            };
+            core.app
+                .fold_snapshot(base_snapshot, app_messages, new_snapshot_index)
                 .await?;
-            self.storage.put_tag(new_snapshot_index, new_tag).await?;
             let new_snapshot = {
                 let mut e = self.storage.get_entry(new_snapshot_index).await?.unwrap();
                 e.command = Command::serialize(&Command::Snapshot {
@@ -1491,37 +1456,31 @@ impl Log {
             unreachable!()
         }
     }
-    async fn run_gc(&self, core: Arc<RaftCore>) -> anyhow::Result<()> {
+    async fn run_gc(&self, core: Arc<RaftCore>) -> Result<()> {
+        let l = self.storage.get_head_index().await?;
         let r = self.get_snapshot_index();
-        log::debug!("gc .. {}", r);
+        log::debug!("gc {}..{}", l, r);
 
-        // Delete old snapshots
-        let ls: Vec<Index> = self
-            .storage
-            .list_tags()
-            .await?
-            .range(..r)
-            .map(|x| *x)
-            .collect();
-        for i in ls {
-            if let Some(tag) = self.storage.get_tag(i).await?.clone() {
-                core.app.delete_snapshot(&tag).await?;
-                self.storage.delete_tag(i).await?;
-            }
-        }
-        // Remove entries
-        self.storage.delete_before(r).await?;
-        // Remove acks
-        let ls: Vec<u64> = self
-            .ack_chans
-            .read()
-            .await
-            .range(..r)
-            .map(|x| *x.0)
-            .collect();
-        for i in ls {
+        for i in l..r {
+            // Remove remaining ack?
+            // Not sure it really exists.
             self.ack_chans.write().await.remove(&i);
+
+            let entry = self.storage.get_entry(i).await?;
+            let entry = entry.unwrap();
+            match Command::deserialize(&entry.command) {
+                Command::Snapshot { .. } => {
+                    // Delete snapshot entry.
+                    // There should be snapshot resource to this snapshot entry.
+                    core.app.delete_snapshot(i).await?;
+                }
+                _ => {}
+            }
+            // Delete the entry.
+            // but after everything is successfully deleted.
+            self.storage.delete_entry(i).await?;
         }
+
         Ok(())
     }
 }
