@@ -1,14 +1,19 @@
 use crate::{Clock, Command, Id, Index, Term};
+use anyhow::Result;
 use bytes::Bytes;
-use std::collections::BTreeSet;
 
 /// In-memory implementation backed by BTreeMap.
 pub mod memory;
 
+/// Persistent implementation backed by normal files.
+pub mod file;
+
 /// Persistent implementation backed by RocksDB.
-#[cfg(feature = "persistency")]
-#[cfg_attr(docsrs, doc(cfg(feature = "persistency")))]
-pub mod disk;
+#[cfg(feature = "rocksdb-backend")]
+#[cfg_attr(docsrs, doc(cfg(feature = "rocksdb-backend")))]
+pub mod rocksdb;
+
+mod persistency;
 
 /// Serialized Ballot.
 /// Ballot is a record of the previous vote.
@@ -34,26 +39,20 @@ pub struct Entry {
     pub(crate) command: Bytes,
 }
 
-/// The abstraction for the backing storage.
+/// The abstraction of the log storage.
 /// Conceptually it is considered as a sequence of log entries and the recent vote.
 #[async_trait::async_trait]
 pub trait RaftStorage: Sync + Send + 'static {
-    /// Delete range ..r
-    async fn delete_before(&self, r: Index) -> anyhow::Result<()>;
-    async fn insert_entry(&self, i: Index, e: Entry) -> anyhow::Result<()>;
-    async fn get_entry(&self, i: Index) -> anyhow::Result<Option<Entry>>;
-    async fn get_last_index(&self) -> anyhow::Result<Index>;
-    async fn save_ballot(&self, v: Ballot) -> anyhow::Result<()>;
-    async fn load_ballot(&self) -> anyhow::Result<Ballot>;
-    async fn put_tag(&self, i: Index, snapshot: crate::SnapshotTag) -> anyhow::Result<()>;
-    async fn delete_tag(&self, i: Index) -> anyhow::Result<()>;
-    async fn get_tag(&self, i: Index) -> anyhow::Result<Option<crate::SnapshotTag>>;
-    async fn list_tags(&self) -> anyhow::Result<BTreeSet<Index>>;
+    async fn insert_entry(&self, i: Index, e: Entry) -> Result<()>;
+    async fn delete_entry(&self, i: Index) -> Result<()>;
+    async fn get_entry(&self, i: Index) -> Result<Option<Entry>>;
+    async fn get_head_index(&self) -> Result<Index>;
+    async fn get_last_index(&self) -> Result<Index>;
+    async fn save_ballot(&self, v: Ballot) -> Result<()>;
+    async fn load_ballot(&self) -> Result<Ballot>;
 }
 
-pub(crate) async fn find_last_snapshot_index<S: RaftStorage>(
-    storage: &S,
-) -> anyhow::Result<Option<Index>> {
+pub(crate) async fn find_last_snapshot_index<S: RaftStorage>(storage: &S) -> Result<Option<Index>> {
     let last = storage.get_last_index().await?;
     for i in (1..=last).rev() {
         let e = storage.get_entry(i).await?.unwrap();
@@ -66,7 +65,7 @@ pub(crate) async fn find_last_snapshot_index<S: RaftStorage>(
 }
 
 #[cfg(test)]
-async fn test_storage<S: RaftStorage>(s: S) -> anyhow::Result<()> {
+async fn test_storage<S: RaftStorage>(s: S) -> Result<()> {
     use crate::Uri;
     use std::collections::HashSet;
 
@@ -84,7 +83,7 @@ async fn test_storage<S: RaftStorage>(s: S) -> anyhow::Result<()> {
     };
 
     // Vote
-    let uri: Uri = "hoge".parse().unwrap();
+    let uri: Uri = "https://192.168.100.98:50001".parse().unwrap();
     let id: Id = uri.into();
     assert_eq!(
         s.load_ballot().await?,
@@ -106,12 +105,6 @@ async fn test_storage<S: RaftStorage>(s: S) -> anyhow::Result<()> {
         }
     );
 
-    // Tag
-    let tag: crate::SnapshotTag = vec![].into();
-    assert!(s.get_tag(10).await?.is_none());
-    s.put_tag(10, tag.clone()).await?;
-    assert_eq!(s.get_tag(10).await?, Some(tag.clone()));
-
     assert_eq!(find_last_snapshot_index(&s).await?, None);
     assert_eq!(s.get_last_index().await?, 0);
     assert!(s.get_entry(1).await?.is_none());
@@ -122,24 +115,38 @@ async fn test_storage<S: RaftStorage>(s: S) -> anyhow::Result<()> {
     let e4 = e.clone();
     let e5 = e.clone();
     s.insert_entry(1, sn1).await?;
+    assert_eq!(s.get_head_index().await?, 1);
     assert_eq!(s.get_last_index().await?, 1);
     assert_eq!(find_last_snapshot_index(&s).await?, Some(1));
     s.insert_entry(2, e2).await?;
     s.insert_entry(3, e3).await?;
     s.insert_entry(4, e4).await?;
     s.insert_entry(5, e5).await?;
+    assert_eq!(s.get_head_index().await?, 1);
     assert_eq!(s.get_last_index().await?, 5);
 
     let sn4 = sn.clone();
     s.insert_entry(4, sn4).await?;
+    assert_eq!(s.get_head_index().await?, 1);
     assert_eq!(find_last_snapshot_index(&s).await?, Some(4));
     let sn2 = sn.clone();
     s.insert_entry(2, sn2).await?;
     assert_eq!(find_last_snapshot_index(&s).await?, Some(4));
 
     assert!(s.get_entry(1).await?.is_some());
-    s.delete_before(4).await?;
+    // delete entries before the last snapshot
+    for i in 1..4 {
+        s.delete_entry(i).await?;
+    }
+    assert_eq!(s.get_head_index().await?, 4);
     assert!(s.get_entry(1).await?.is_none());
 
+    for i in 6..=1000 {
+        let ei = e.clone();
+        s.insert_entry(i, ei).await?;
+    }
+    assert_eq!(s.get_head_index().await?, 4);
+    assert_eq!(s.get_last_index().await?, 1000);
+
     Ok(())
 }
